import ecs_logging
import os
import warnings
from typing import Any, Union
from datetime import datetime


class FasalStdlibFormatter(ecs_logging.StdlibFormatter):
    def __init__(self, datefmt=None, extra=None, exclude_fields=()):
        
        # Keep required keys and default values
        self._required = {
            'severity': '',
            'source': '',
            'env': os.getenv('ENV', ''),
            'type': 'app-log',
            'message': '',
            'label': ''
        }

        # Rename any key with custom field
        # nested dict can be renamed with dot operator
        self._rename_field = {
            'severity': 'log.level',
        }

        # Copy a filed value without deleting
        self._copy_field = {
            "source": 'log.origin.file.name'
        }

        if extra is not None:
            self._required.update(extra)
        super().__init__(
            datefmt = datefmt,
            exclude_fields = exclude_fields,
            extra = extra
        )
    
    def _rename(self, source, ref_dict, keep_keys, drop=True):
        for key in self._required.keys():
            if key in ref_dict.keys():
                try:
                    source[key] = source[ref_dict.get(key)]
                    if drop: del source[ref_dict.get(key)]
                except KeyError:
                    pass

            # If key already exists, remove the default value
            if key in source.keys():
                try:
                    keep_keys.remove(key)
                except ValueError:
                    pass
        return keep_keys, source

    def format_to_ecs(self, record):

        result = super().format_to_ecs(record)
        result = ecs_logging._utils.flatten_dict(result)

        # rename the result
        default_keys = list(self._required.keys())
        default_keys, result = self._rename(result, ref_dict=self._rename_field, keep_keys=default_keys, drop=True)

        # Copy fields from result
        default_keys, result = self._rename(result, ref_dict=self._copy_field, keep_keys=default_keys, drop=False)

        result.update({key: self._required[key] for key in default_keys})
        return ecs_logging._utils.normalize_dict(result)


class SlackNotification():
    def __init__(self, webhook: Any = None, DEV:bool = True):
        self._dev = os.getenv('DEV', DEV)
        if webhook is None:
            warnings.warn("Using a default slack channel", stacklevel=2)
            webhook = 'https://hooks.slack.com/services/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'
        self._webhook = webhook

    def notify(self, message):
        if not self._dev:
            try:
                from slack_sdk import WebhookClient
            except ImportError:
                warnings.warn("You are in development mode. \nSet $DEV=True", stacklevel=2)
                pass
            try:
                webhook = WebhookClient(self._webhook)
                response = webhook.send(
                    text="custom trigger",
                    blocks=[
                        {   
                            "type": "header",
                            "text": {
                                "type": "plain_text",
                                "text": "Code Anomaly :expressionless:",
                                "emoji": True
                            }
                        },
                        {
                            "type": "section",
                            "fields": [
                                {
                                    "type": "mrkdwn",
                                    "text": "*Type:*\nWebhook trigger"
                                },
                                {
                                    "type": "mrkdwn",
                                    "text": f'*Timestamp:*\n{datetime.now().strftime("%m/%d/%Y, %H:%M:%S")}'
                                }
                            ]
                        },
                        {
                            "type": "section",
                            "text": {
                                "type": "plain_text",
                                "text": f"*Message*\n{message}"
                            }
                        }
                    ]
                )
            except Exception as err:
                warnings.warn(f'Need a webhook to send a message \n {err}', stacklevel=2)
                pass



import os
import http
import warnings
import json
import yaml
import logging
import logging.config
import ecs_logging

from http import client
from functools import partial
from typing import Union
from fasal_logger._logger import SlackNotification, FasalStdlibFormatter

class LoggerInitializer():
    def __call__(self, logger, config):
        config = os.path.abspath(config)
        self._configure(config)
        http.client.HTTPConnection.debuglevel = 1
        http_print = partial(self._print_to_log, logger)
        http.client.print = http_print

    def _configure(self, config):
        if not os.path.exists(config):
            raise FileNotFoundError(f"Check if the .yml file is present in this path or not ({config})")
        with open(config, 'r') as f:
            _config = yaml.safe_load(f.read())
            _config = self._expand_vars(_config)
            logging.config.dictConfig(_config)

    def _expand_vars(self, config_var) -> Union[dict, str]:
        def expand(config_var) -> str:
            if isinstance(config_var, str):
                config_var = os.path.expandvars(config_var)
                if '$' in config_var:
                    warnings.warn("environment variable parsing failed", stacklevel=2)
            return config_var
        config = expand(config_var)
        if isinstance(config_var, dict):
            _config_var = json.dumps(config_var)
            config_var = json.loads(expand(_config_var))

        return config_var

    def _print_to_log(self, logger, *args, **kwargs):
        k = ".".join(args[:-1])
        v = args[-1]
        extra = ecs_logging._utils.de_dot(k,v)
        extra.update(kwargs)
        extra.update({'type': 'access-log'})
        logger.debug("HTTP log", extra=extra)





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fasal_logger']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML==6.0', 'ecs-logging==2.0.0', 'slack-sdk==3.18.1']

setup_kwargs = {
    'name': 'fasal-logger',
    'version': '0.0.2',
    'description': 'json based python logger with support for slack notification',
    'long_description': '\n### Installation\nInstall the package using the following command\n```bash\npip install fasal-logger\n```\n\n--------------\n### Configuration\n- Create a file `logger.yml`. Copy the contents from the repository and make the necessary changes (If needed)\n- The logger defined here defaults to console. In order to log the contents into a file, make the required changes to the `logger.yml`.\n``` yaml\nroot:\n  handlers: [console, file]\n\nloggers:\n  fasalLogger:\n    handlers: [console, file]\n\n```\n\nAnd change the filename and location.\n```yaml\nhandler:\n    file:\n        filename: \'logging.example.log\'\n```\n\n### Usage\nIn order to use the logger in your code, add the following piece of code at the top of your .py file\n\n**Note:** `Get the webhook from the infra team to able to send messages to slack to any other channel. (By default using #fasal-ai-infra)`\n\n```python\nimport logging\nimport logging.config\nimport os\nfrom fasal_logger import LoggerInitializer, SlackNotification\n\nlogging.captureWarnings(True)\nlogger = logging.getLogger(__name__)\nlogger_init = LoggerInitializer()\nlogger_init(logger=logger, config=\'./fasal_logger/logger.yml\')\nslk = SlackNotification() # set parameter for webhook, DEV (if needed)\n\n# Use logger now\nlogger.info("Logger set")\n\n# Send a message to slack channel\nslk.notify(message="testing")\n\n```\n----------\n\n**Variables taken from environment are:**\n\n  - `SLACK_WEBHOOK`: Channel webhook trigger\n  - `DEV`: If True, no message is send to slack\n  - `ENV`: logger environment (staging/production/development)\n\n------------\nBuild and Published using (`poetry`)[https://python-poetry.org/docs/cli/#publish]',
    'author': 'Binay Pradhan',
    'author_email': 'binay.pradhan@wolkus.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
